package Plugins::MyVOCO::Plugin;

# Plugin to handle all MyVOCO transactions
#
use Slim::Utils::Log;
use Slim::Utils::Prefs;


my $prefs = preferences('plugin.myvoco');

my $log = Slim::Utils::Log->addLogCategory( {
	category     => 'plugin.myvoco',
	defaultLevel => 'WARN',
	description  => 'Plugin myvoco',
} );

use Plugins::MyVOCO::ProtocolHandler;
use Plugins::MyVOCO::ProtocolHandlerOverride;

sub initPlugin {
	my $class = shift;
	
	$log->error('(initPlugin) MyVOCO is starting up!!');

	Slim::Player::ProtocolHandlers->registerHandler(
		voco => 'Plugins::MyVOCO::ProtocolHandler'
	);
	Slim::Player::ProtocolHandlers->registerHandler(
		vococ => 'Plugins::MyVOCO::ProtocolHandlerOverride'
	);
	
	Slim::Networking::Discovery::addTLVHandler({
		'VMSV' => sub { get_version() },
	});
	
	$log->error('(initPlugin) ProtocolHandler registered.');
	
	#        |requires Client
	#        |  |is a Query  
	#        |  |  |has Tags 
	#        |  |  |  |Function to call
	#        C  Q  T  F 
	
	Slim::Control::Request::addDispatch(['myvoco', 'version', '?'], [0, 1, 0, \&check_version]);

}

sub check_version {
	my $request = shift;

	# check this is the correct query.
	if ($request->isNotQuery([['myvoco', 'version']])) {
		$request->setStatusBadDispatch();
		return;
	}
	
	# Just Return Hardcoded SVN Version number, this will get replaced at build time
	$request->addResult('_version', get_version());
	$request->setStatusDone();
}

sub get_version {
	return '19441';
}

