package Plugins::MyVOCO::ProtocolHandler;

use strict;

use Slim::Utils::Log;
use Slim::Utils::Prefs;
use Slim::Utils::Timers;
use Slim::Schema::RemoteTrack;
use Slim::Schema::RemotePlaylist;

use Plugins::MyVOCO::Utils;

use Data::Dumper;
use JSON::XS::VersionOneAndTwo;
use LWP::Simple;
use URI;
use Time::HiRes;

use base qw(Slim::Player::Protocols::HTTP);


#TODO almost all calles to vzone need to be async
# see methods that have callbacks 
# see examples in original slim source that call squeezenetwork

#TODO review all callbacks for sync processing
# currently calling the master ip address helps but many of the callbacks
# should also supress multiple callback for the same event by supressing 
# if the master was not the one requested, i.e  onStream,onPlayout, onStop and others

my $log   = logger('plugin.myvoco');
my $prefs = preferences('plugin.myvoco');

# Other method know to exist not nit implimented
# scanStream         : Utils::Scanner::Remote
# trackInfoURL       : Menu::TrackInfo
# isPlaylistURL      : Music::Info
# trackGain          : Player::ReplayGain
# getIcon            : Player::ProtocolHandlers
# canHandleTranscode : Player::Song
# updateOnStream     : Player::StreamingController

my $service = '';
my $remote = 0;


sub isRemote { 

	$log->debug('****************** is remote ************************') ;
	# return 0 for live streams that should stop on pause and restart on unpause
	# return 1 for track at a time streams that are able to pause or unpause 

	my ($class, ) = @_;
	
	$log->debug('remote is currently [' . $remote . ']');
	
	# For now just return 0 so that we can test this
	return $remote; 
}

sub canSeek {

	#
	# # Can only seek if bitrate and duration are known
	#
	my ($class, $player, $song) = @_;
	
	my $url = $song->track()->url;
	$log->debug('canSeek calling cache/remote');
	my $control = Plugins::MyVOCO::Utils::getFromCacheOrRemote('player_control', $player, $url, 1);

	if (defined $control->{canSeek}) {
		$log->info('seems like we have the canSeek value! [' . $control->{canSeek} . ']');
		return $control->{canSeek};
	} else {
		$log->error('we have a player_control but not the canSeek key, default to 0');
	}

	return 0;
}

#
# getSeekData and getSeekDataByPosition are overides of HTTP.pm
#   at the moment we do not have anything better then the existing methods 
#   note that the existing methods are very crude and will not work in all cases  
#
#sub getSeekDataByPosition { 
#	$log->info('called');
#	#TODO not sure; study slim examples
#	undef;
#}
#sub getSeekData { 
#	$log->info('called');
#	#TODO not sure; study slim examples
#	return undef;
#}

sub isAudioURL () {
	$log->debug('called');
	#TODO not sure; study slim examples
	return 1;
}

# Interesting method found in another plugin  could come in handy
# default buffer 3 seconds of 256kbps audio
#sub bufferThreshold {
#	return 32 * ($prefs->get('bufferSecs') || 3); 
#}

sub onStream {
	my ( $class, $player, $song ) = @_;

	my $originalMaster = $song->pluginData('client');
	if ($originalMaster == $player) {
		my $ip = $originalMaster->ip();
		my $url = $song->track()->url;
		Plugins::MyVOCO::Utils::callVzcAsync($ip, 'onStream', $url, sub { $log->debug('Stream cb')} , sub { $log->debug('Stream ecb')});
	} else {
		$log->warn("not calling for player [" . $player->id() . "]");
	}
}

sub playpointCallback {
	my $client = shift;
	my $song = shift;
	my $currentSong = $client->playingSong();
	my $originalMaster = $song->pluginData('client');
	my $t = Time::HiRes::time();
	$log->warn("old song was [$song]");
	$log->warn("new song is [$currentSong]");
	if ($originalMaster->isPlaying and $song eq $currentSong) {
		$log->info("song is still playing!");
		my $ip = $originalMaster->ip();
		my $url = $song->track()->url;
		Plugins::MyVOCO::Utils::callVzcAsync(
				$ip, 'on30sec', $url,
				sub { $log->debug('playpoint cb')},
				sub { $log->error('playpoint callback error!')}
			);
	} else {
		$log->info("song changed or is not playing anymore");
	}
	return;
}

sub onPlayout{
	my ($class, $song, $self ) = @_;
	$log->debug('Got onPlayout');
	my $originalMaster = $song->pluginData('client');
	my $ip = $originalMaster->ip();
	my $url = $song->track()->url;
	Plugins::MyVOCO::Utils::callVzcAsync($ip, 'onPlayout', $url, sub { $log->debug('Playout cb')} , sub { $log->error('Playout callback error!')});
}

sub onStop{
	my ($class, $song) = @_;
	$log->debug('Got onStop');
	my $originalMaster = $song->pluginData('client');
	my $ip = $originalMaster->ip();
	my $url = $song->track()->url;
	Plugins::MyVOCO::Utils::callVzcAsync($ip, 'onStop', $url, sub { $log->debug('Stop cb')} , sub { $log->error('Stop cb error')});
}


sub canDoAction{
	my ( $class, $client, $url, $action ) = @_;
	# this could be very usfull for things like audioin where pause is bad
	#
	# This only responds with a 1 or a 0 according to the appropriate capabilities of the service
	# From http://wiki.slimdevices.com/index.php/Design_of_new-streaming
	#	'rew'   - restart the current track,
	#	'pause' - pause the stream without closing it,
	#	'stop'  - a proxy for skip, meaning skip to the next track (provided by this handler).
	
	$log->info('Got canDoAction for [' . $action . ']');

	# We are assuming we got this information in the cache, we get it in getNextTrack in player_control

	$log->debug('canDoAction calling cache/remote');
	my $control = Plugins::MyVOCO::Utils::getFromCacheOrRemote('player_control', $client, $url, 1);
	my $actions = $control->{canDoAction} || 0;   # if canDoAction was not provided, return a 0
	my $this_action = $actions->{$action} || 0; # if this action is not listed, return a 0

	if (defined $control->{canDoAction}) {
		$log->debug('canDoAction has ['. Dumper($control->{canDoAction}) . ']');
		my $act = $control->{canDoAction}->{$action};
		if ($act->{callback}) {
			$log->debug('callback!');
			my $ip = $client->ip();
			$url = $url . '&a=' . $action;
			my $response = Plugins::MyVOCO::Utils::getVzcResponse($ip, 'canDoAction', $url);
			my $canDoAction = decode_json($response)->{result};
			$log->debug('response from vzc [' . $canDoAction . ']');
			return $canDoAction;
		} else {
			$log->debug('no callback!');
			# Use the information from player_control object
			if ($act->{canDo}) {
				$log->debug('canDo!');
				return 1;
			} else {
				$log->debug('no canDo!');
				return 0;
			}
		}
	}

	$log->warn('canDoAction was not available!!! defaulting to 1');
	return 1;
}

sub getFormatForURL () { 
	my $classOrSelf = shift;
	my $url = shift;
	my $pUrl = URI->new($url, 'voco');
	$log->debug('[format] url [' . $url . ']');
	my %query = $pUrl->query_form();
	my $format = $query{'format'};
	$log->debug('[format] queryf [' . $query{'format'} . '] form [' . $format . ']');
	if (defined $format) {
		$log->debug('format included in the query! returning [' . $format . ']');
		return $format;
	} else {
		my $service = substr($pUrl->path(), 1); # substr($, 1) removes the preceding '/' from path
		$log->debug('format not included in the query! assuming mp3');
		$log->debug('[format] service [' . $service . '] format [mp3]');
		return 'mp3';
	}
}

# Don't allow looping if the tracks are short
sub shouldLoop () { 
	$log->debug('called');
	return 0;
}

# old comment said:
# # 1 => player-proxy-streaming necessary for sync
# setting to 0, because we don't have or want player proxy
sub usePlayerProxyStreaming { 
	$log->debug('called');
	return 0;
} 


# Avoid scanning
sub scanUrl {
	my ($class, $url, $args) = @_;

	#TODO if the url real url endswith a playlist extension
	# or if we do not have the real play url at this point then if &isPlaylist=True
	# then scan otherwise just call the sucess callback

	$args->{song}->pluginData('client', $args->{client}); # store the client (master) that was active when we first got here
	$args->{cb}->($args->{song}->currentTrack());
}

sub getNextTrack {
	my ($class, $song, $successCb, $errorCb) = @_;

	$log->warn('[getNextTrack] start');

	my $originalMaster = $song->pluginData('client');

	my $url    = $song->currentTrack()->url;
	my $pUrl   = URI->new($url, 'voco');

	Plugins::MyVOCO::Utils::callVzcAsync($originalMaster->ip(), 'url', $url, \&gotNextTrack, \&gotNextTrackError, {
			client        => $originalMaster,
			url           => $url,
			song          => $song,
			callback      => $successCb,
			errorCallback => $errorCb,
		});
}

sub gotNextTrack {
	my $http     = shift;
	my $params   = $http->params;
	my $song		= $params->{song};
	my $successCb 	= $params->{callback};
	my $client		= $params->{client}; # this is actually the originalMaster (as loaded in the callVzcAsync above)
	my $url		= $params->{url};

	my $content = $http->content;
	my $content_json = decode_json($content);
	my $streamUrl = $content_json->{metadata}->{url};

	# We are assuming that the URL that we got was fixed by scanUrl
	$song->streamUrl($streamUrl);

	my $cache  = Slim::Utils::Cache->new('voco');
	# if I did this right $client->id isthe mac address of the master player
	#  needed so that the correct data can be looked up latter even if it is in a sync group
	$cache->set( 'voco_url_meta' . $client->id . $url, $content_json->{metadata}, 86400 );
	$cache->set( 'voco_url_control' . $client->id . $url, $content_json->{player_control}, 3600 );

	$successCb->();
}

sub gotNextTrackError {
	my $http     = shift;
	my $params   = $http->params;
	my $errorcb = $params->{errorCallback};
	$log->error('error!! [' .  $http->error . ']');
	$errorcb->('PLUGIN_MYVOCO_ERROR', $http->error);
}

sub formatOverride {
	my $self = shift;
	my $song =  shift;
	my $originalMaster = $song->pluginData('client');

	my $url = $song->track()->url;
	$log->debug('formatOverride calling cache/remote');
	my $playercontrol = Plugins::MyVOCO::Utils::getFromCacheOrRemote('player_control', $originalMaster, $url, 1);
	$log->debug('formatOverride playercontrol ['.$playercontrol.']');
	if ($playercontrol->{getFormatForURL}) {
		$log->debug('we have a format for this url!');
		my $format = $playercontrol->{getFormatForURL};
		$log->debug('format ['.$format.']');
		return $format;
	}
	$log->debug("we DON'T have a format for this url! Let's say it's MP3!");
	return 'mp3';
}

sub bufferThreshold { 
	$log->debug('bufferThreshold called');
	my $self = shift;
	my $client =  shift;
	my $realUrl = shift;
	my $song = $client->streamingSong();
	$log->debug('bufferThreshold calling cache/remote');
	my $playercontrol = Plugins::MyVOCO::Utils::getFromCacheOrRemote('player_control', $client, $song->track()->url, 1);
	if ($playercontrol->{bufferThreshold}) {
		$log->debug('bufferThreshold found in player control ['.$playercontrol->{bufferThreshold}.']');
		return $playercontrol->{bufferThreshold};
	}
	$log->debug('bufferThreshold not found in player control, defaulting to [80]');
	return 80;
 }

sub forceBufferThreshold {
	my $self = shift;
	my $client =  shift;
	my $realUrl = shift;	
	return $self->bufferThreshold($client, $realUrl);
}

sub isRepeatingStream {
	# Magic method 1=call get next track after playing current stream
	#              0=done is done
	my $class = shift;
	my $song  = shift;

	my $url = $song->track()->url;
	my $pUrl   = URI->new($url, 'voco');
	my %query = $pUrl->query_form();

	if ($query{'repeat'}) {
		$log->debug('repeat was found in query, with value [' .  $query{'repeat'} . ']');
		return 1;
	}
	$log->debug('repeat not included in url, assuming single track');
	return 0;
}

sub getMetadataFor {
	my ( $class, $client, $url, $forceCurrent ) = @_;

	if (not $client) {
		$log->warn('we are not getting the client for this call! for now, return an empty hash');
		return {};
	}

	my $song = $forceCurrent ? $client->streamingSong() : $client->playingSong();
	return {} unless $song;

	my $originalMaster = $song->pluginData('client');
	if ($originalMaster) {
		$client = $originalMaster; # if we have the original master, replace client with it
	}
	$log->debug('getMetadataFor calling cache/remote');
	my $metadata = Plugins::MyVOCO::Utils::getFromCacheOrRemote('metadata', $client, $url, 0); ### <-non-blocking
	
	if (defined $metadata) {
		$log->debug($metadata);
		return $metadata;
	} else {
		$log->error('could not obtain metadata!');
		return {};
	}
}


sub new {
  my $class  = shift;
  my $args   = shift;

  #TODO we need a way to do ssl
  
  $log->warn( 'MyVOCO Protocol Handler object initialization');

  my $client = $args->{client};
  
  my $song      = $args->{song};
  my $url        = $song->track()->url;
  my $streamUrl = $song->streamUrl() || return;
  my $track     = $song->pluginData();
  
  $log->info( 'MyVOCO original URL: [' . $url . ']');
  $log->info( 'MyVOCO PH got this URL: [' . $streamUrl . ']');

  $log->debug('new calling cache/remote (pc)');
  my $playercontrol = Plugins::MyVOCO::Utils::getFromCacheOrRemote('player_control', $client, $url, 1);
  $log->debug('new calling cache/remote (meta)');
  my $metadata = Plugins::MyVOCO::Utils::getFromCacheOrRemote('metadata', $client, $url, 1);
  
  my $sock = $class->SUPER::new( {
    url     => $streamUrl,
    song    => $song,
    client  => $client,
    bitrate => $metadata->{bitrate_raw},
  } ) || return;
  
  ${*$sock}{contentType} = 'audio/mpeg';

  ### ============== Start timer ==================
  ### TODO: This is not the best place to do the timer starting.
  ### We want to do it when playback actually starts, after the buffering.
  ### Meanwhile, we are setting the playpoint callback a little bit later, at 33 instead of 30 secs.
  
  ### playpoint value in player_control can override the time of the callback
  my $delay = $playercontrol->{playpoint} || 33; 
  $log->debug("playpoint to report [$delay]");

  my $t = Time::HiRes::time();
  my $timer = Slim::Utils::Timers::setTimer(
    $client,
    $t + $delay,
    \&playpointCallback,
    $song );
  ### ========= Timer code finishes here =========

  $remote = $playercontrol->{isRemote} || 0;
  $log->debug("remote [$remote]");

  return $sock;
};


1;
