package Plugins::NavvoFormats::Plugin;

use strict;

use base qw(Slim::Plugin::Base);

use Slim::Control::Request;
use Slim::Utils::Log;
use Slim::Utils::Prefs;

my $subscribed = 0;

my $log = Slim::Utils::Log->addLogCategory({
	'category'		=> 'plugin.navvo.formats',
	'defaultLevel'	=> 'INFO',
	'description'	=> 'Navvo Formats Plugin',
	});

my $prefs = preferences('plugin.navvo.formats');
my $prefsSrv = preferences('server');

sub getDisplayName {
	return 'Navvo Formats Plugin';
}

sub initPlugin {
	my $class = shift;
	
	$log->info("initializing Navvo Formats plugin");

#        |requires Client
#        |  |is a Query
#        |  |  |has Tags
#        |  |  |  |Function to call
#        C  Q  T  F	
	
	Slim::Control::Request::addDispatch(['navvo_nativeformats', '?'], 
        [1, 1, 0, \&nativeFormats]);

	Slim::Control::Request::addDispatch(['navvo_radioformats', '?'], 
        [1, 1, 0, \&radioFormats]);

}

sub nativeFormats {
	my $request = shift;
	$log->info("--- *** --- native formats --- *** ---");
	
	my $client = $request->client();
	
	#my $param1 = $request->getParam("_param");

    my @clformats = $client->formats;
    
    $log->info("player formats: [" . join(" ",@clformats) . "]");	
#	$request->addResult('_playerformats', join(" ",@formats));	

	my $cnt = 0;
	$request->addResult('count', scalar(@clformats));
	foreach my $format (@clformats) {
		$request->addResultLoop('format_loop', $cnt, 'format', $format);
		$cnt++;
	}
	
	$request->setStatusDone();
}

sub radioFormats {
	my $request = shift;
	$log->info("--- *** --- radio formats --- *** ---");
	
	my $client = $request->client();


		# In order of preference
	tie my %rtFormats, 'Tie::IxHash', (
		aac     => 'aac',
		ogg     => 'ogg',
		mp3     => 'mp3',
		wmpro   => 'wmap',
		wma     => 'wma',
		wmvoice => 'wma',
		# Real Player is supported through the AlienBBC plugin
		real    => 'rtsp',
	);

	my @formats = keys %rtFormats;
	my $id = '';
	
	if ($client) {
		my %playerFormats = map { $_ => 1 } $client->formats;
	
		# RadioTime's listing defaults to giving us mp3 and wma streams only,
		# but we support a few more
		@formats = grep {
		
			# format played natively on player?
			my $canPlay = $playerFormats{$rtFormats{$_}};
				
			if ( !$canPlay && main::TRANSCODING ) {
	
				foreach my $supported (keys %playerFormats) {
					
					if ( Slim::Player::TranscodingHelper::checkBin(sprintf('%s-%s-*-*', $rtFormats{$_}, $supported)) ) {
						$canPlay = 1;
						last;
					}
	
				}
			}
	
			$canPlay;
	
		} keys %rtFormats;
		
		$id = $client->uuid || $client->id;
	}

	#my $rtForm = join(',', @formats);

	#$request->addResult('rtFormats', $rtForm);

	my $cnt = 0;
	$request->addResult('count', scalar(@formats));
	foreach my $format (@formats) {
		$request->addResultLoop('format_loop', $cnt, 'format', $format);
		$cnt++;
	}
	
	$request->setStatusDone();

}

1;
