package Plugins::NavvoInit::Plugin;

use strict;

use base qw(Slim::Plugin::Base);

use Slim::Control::Request;
use Slim::Utils::Log;
use Slim::Utils::Prefs;

my $subscribed = 0;

my $log = Slim::Utils::Log->addLogCategory({
	'category'		=> 'plugin.navvo.init',
	'defaultLevel'	=> 'INFO',
	'description'	=> 'Navvo Init Plugin',
	});

my $prefs = preferences('plugin.navvo.init');
my $prefsSrv = preferences('server');
my $prefsLast = preferences('plugin.slimscrobbler');

sub getDisplayName {
	return 'Navvo Init Plugin';
}

sub initPlugin {
	my $class = shift;
	
	$log->info("initializing Navvo Init plugin");
	
	## Setting number of sockets
	
	my $maxSockets = $prefsSrv->get('tcpConnectMaximum');
	
	if ($maxSockets < 120) {
		$prefsSrv->set('tcpConnectMaximum', 120);	
	}
	
	## Disabling inotify
	
	$prefs->set('disable_socket_killer', 0);

#        |requires Client
#        |  |is a Query
#        |  |  |has Tags
#        |  |  |  |Function to call
#        C  Q  T  F	
	
	Slim::Control::Request::addDispatch(['navvo_lastfmsetup', '_username', '_passhash'],
		[1, 0, 0, \&setupLastFM]);
	
	Slim::Control::Request::addDispatch(['navvo', 'check_clients'],
		[0, 1, 0, \&check_clients]);

}


sub check_clients {
	my $request = shift;
	Slim::Networking::Slimproto::verify_check_timer();
	$request->addResult('Result', 'Ran verify_check_timer');
	$request->setStatusDone();
}


sub setupLastFM {
	my $request = shift;
	$log->info(".received navvo_lastfmsetup command");
	
	my $username = $request->getParam('_username');
	my $passhash = $request->getParam('_passhash');
	my $client = $request->client;

	if ($username and $passhash) {

		my $accHashRef = $prefsLast->get('accounts');
		$log->info(".reading accounts ref [" . $accHashRef . "]");

		if (exists($accHashRef->{$username})) {
			# The username has been included, check if the passhash is the latest
			$log->info(".username exists [" . $username . "]");
			if ($accHashRef->{$username} ne $passhash) {
				$log->info(".username has different pass, updating [" . $username . "]");
				# The hash is different from the one provided, update.
				$accHashRef->{$username} = $passhash;
			} else {
				$log->info(".username has same pass [" . $username . "]");
				# Nothing happens here, the passhash is up to date.
			}
		} else {
			# The username has not been included, include it.
			$log->info(".username doesn't exists, creating [" . $username . "]");
			$accHashRef->{$username} = $passhash;
		}

		my $currentIDforPlayer = $prefsLast->client($client)->get('userid');
		if ($currentIDforPlayer ne $username) {
			$log->info(".username is not selected, selecting [" . $username . "]");
			$prefsLast->set('accounts', $accHashRef);
			$prefsLast->client($client)->set('userid', $username);
		} else {
			$log->info(".username had already been selected [" . $username . "]");
		}
		$request->setStatusDone();
	}

}


1;
